@@ -59,16 +59,49 @@ static EVENT_HANDLER(APPLICATION_LAUNCHED)
     if (!workspace_application_is_observable(process)) {
         debug("%s: %s (%d) is not observable, subscribing to activationPolicy changes\n", __FUNCTION__, process->name, process->pid);
         workspace_application_observe_activation_policy(g_workspace_context, process);
-        return;
+
+        //
+        // NOTE(koekeishiya): Do this again in case of race-conditions between the previous check and key-value observation subscription.
+        // Not actually sure if this can happen in practice..
+        //
+
+        if (workspace_application_is_observable(process)) {
+            @try {
+                NSRunningApplication *application = __atomic_load_n(&process->ns_application, __ATOMIC_RELAXED);
+                if (application && [application observationInfo]) {
+                    [application removeObserver:g_workspace_context forKeyPath:@"activationPolicy" context:process];
+                }
+            } @catch (NSException * __unused exception) {}
+        } else { return; }
     }
 
     if (!workspace_application_is_finished_launching(process)) {
         debug("%s: %s (%d) is not finished launching, subscribing to finishedLaunching changes\n", __FUNCTION__, process->name, process->pid);
         workspace_application_observe_finished_launching(g_workspace_context, process);
-        return;
+
+        //
+        // NOTE(koekeishiya): Do this again in case of race-conditions between the previous check and key-value observation subscription.
+        // Not actually sure if this can happen in practice..
+        //
+
+        if (workspace_application_is_finished_launching(process)) {
+            @try {
+                NSRunningApplication *application = __atomic_load_n(&process->ns_application, __ATOMIC_RELAXED);
+                if (application && [application observationInfo]) {
+                    [application removeObserver:g_workspace_context forKeyPath:@"finishedLaunching" context:process];
+                }
+            } @catch (NSException * __unused exception) {}
+        } else { return; }
     }
 
-    struct application *application = application_create(process);
+    //
+    // NOTE(koekeishiya): If we somehow receive a duplicate launched event due to the subscription-timing-mess above,
+    // simply ignore the event..
+    //
+
+    struct application *application = window_manager_find_application(&g_window_manager, process->pid);
+    if (application) { return; } else { application = application_create(process); }
+
     if (!application_observe(application)) {
         bool ax_retry = application->ax_retry;
 
@@ -144,7 +177,9 @@ static EVENT_HANDLER(APPLICATION_LAUNCHED)
         }
 
 next:
-        event_signal_push(SIGNAL_WINDOW_CREATED, window);
+        if (window_manager_is_window_eligible(window)) {
+            event_signal_push(SIGNAL_WINDOW_CREATED, window);
+        }
     }
 
     //
@@ -211,7 +246,10 @@ static EVENT_HANDLER(APPLICATION_TERMINATED)
 
         if (g_mouse_state.window == window) g_mouse_state.window = NULL;
 
-        event_signal_push(SIGNAL_WINDOW_DESTROYED, window);
+        if (window->is_eligible) {
+            event_signal_push(SIGNAL_WINDOW_DESTROYED, window);
+        }
+
         window_manager_remove_window(&g_window_manager, window->id);
         window_unobserve(window);
         window_destroy(window);
@@ -444,7 +482,9 @@ static EVENT_HANDLER(WINDOW_CREATED)
         window_manager_add_managed_window(&g_window_manager, window, view);
     }
 
-    event_signal_push(SIGNAL_WINDOW_CREATED, window);
+    if (window_manager_is_window_eligible(window)) {
+        event_signal_push(SIGNAL_WINDOW_CREATED, window);
+    }
 }
 
 static EVENT_HANDLER(WINDOW_DESTROYED)
@@ -465,7 +505,10 @@ static EVENT_HANDLER(WINDOW_DESTROYED)
 
     if (g_mouse_state.window == window) g_mouse_state.window = NULL;
 
-    event_signal_push(SIGNAL_WINDOW_DESTROYED, window);
+    if (window->is_eligible) {
+        event_signal_push(SIGNAL_WINDOW_DESTROYED, window);
+    }
+
     window_manager_remove_window(&g_window_manager, window->id);
     window_destroy(window);
 }
@@ -1010,11 +1053,7 @@ static EVENT_HANDLER(MOUSE_MOVED)
     struct window *window = wid ? window_manager_find_window(&g_window_manager, wid) : window_manager_find_window_at_point(&g_window_manager, point);
     if (window) {
         if (window->id == g_window_manager.focused_window_id) goto out;
-
-        if (!window_rule_check_flag(window, WINDOW_RULE_MANAGED)) {
-            if (!window_level_is_standard(window))                            goto out;
-            if ((!window_is_standard(window)) && (!window_is_dialog(window))) goto out;
-        }
+        if (!window_manager_is_window_eligible(window))       goto out;
 
         if (g_window_manager.ffm_mode == FFM_AUTOFOCUS) {
 
@@ -1070,8 +1109,9 @@ static EVENT_HANDLER(MOUSE_MOVED)
                     struct window *sub_window = window_manager_find_window(&g_window_manager, wid);
                     if (!sub_window) continue;
 
-                    if (!window_check_flag(sub_window, WINDOW_FLOAT))               continue;
-                    if (window_level(sub_window->id) != g_layer_below_window_level) continue;
+                    if (!window_check_flag(sub_window, WINDOW_FLOAT))                     continue;
+                    if (window_level(window->id) != window_level(sub_window->id))         continue;
+                    if (window_sub_level(window->id) != window_sub_level(sub_window->id)) continue;
 
                     if (CGRectContainsRect(window->frame, sub_window->frame)) {
                         occludes_window = true;
@@ -1092,7 +1132,8 @@ static EVENT_HANDLER(MOUSE_MOVED)
         CGRect menu = display_manager_menu_bar_rect(cursor_did);
         if (cgrect_contains_point(menu, point)) goto out;
 
-        display_manager_focus_display_with_point(cursor_did, point, false);
+        int wid = display_manager_focus_display_with_point(cursor_did, point, false);
+        g_mouse_state.ffm_window_id = wid;
     }
 
 out:
