@@ -1,7 +1,9 @@
 #include <Carbon/Carbon.h>
 #include <Cocoa/Cocoa.h>
+#include <CoreVideo/CoreVideo.h>
 #include <objc/objc-runtime.h>
-#include <mach/mach_time.h>
+#include <mach-o/dyld.h>
+#include <mach-o/swap.h>
 
 #include <stdio.h>
 #include <stddef.h>
@@ -30,6 +32,7 @@
 #include <pthread.h>
 #include <pwd.h>
 #include <spawn.h>
+#include <libproc.h>
 
 #include "misc/extern.h"
 #include "misc/macros.h"
@@ -39,6 +42,7 @@
 #include "misc/notify.h"
 #include "misc/log.h"
 #include "misc/helpers.h"
+#include "misc/macho_dlsym.h"
 #include "misc/sbuffer.h"
 #define HASHTABLE_IMPLEMENTATION
 #include "misc/hashtable.h"
