@@ -1,4 +1,22 @@
 extern struct event_loop g_event_loop;
+extern enum mission_control_mode g_mission_control_mode;
+
+enum mission_control_mode
+{
+    MISSION_CONTROL_MODE_INACTIVE           = 0,
+    MISSION_CONTROL_MODE_SHOW               = 1,
+    MISSION_CONTROL_MODE_SHOW_ALL_WINDOWS   = 2,
+    MISSION_CONTROL_MODE_SHOW_FRONT_WINDOWS = 3,
+    MISSION_CONTROL_MODE_SHOW_DESKTOP       = 4
+};
+
+static const char *mission_control_mode_str[] = {
+    [MISSION_CONTROL_MODE_INACTIVE]           = "inactive",
+    [MISSION_CONTROL_MODE_SHOW]               = "show",
+    [MISSION_CONTROL_MODE_SHOW_ALL_WINDOWS]   = "show-all-windows",
+    [MISSION_CONTROL_MODE_SHOW_FRONT_WINDOWS] = "show-front-windows",
+    [MISSION_CONTROL_MODE_SHOW_DESKTOP]       = "show-desktop"
+};
 
 static struct {
     AXUIElementRef ref;
@@ -58,3 +76,8 @@ void mission_control_unobserve(void)
         CFRelease(g_mission_control_observer.ref);
     }
 }
+
+static inline bool mission_control_is_active(void)
+{
+    return g_mission_control_mode != MISSION_CONTROL_MODE_INACTIVE;
+}
