@@ -27,6 +27,7 @@ static const char *process_name_blacklist[] =
     "chronod",
     "universalaccessd",
     "softwareupdated",
+    "familycircled",
     "Notification Centre",
     "CoreServicesUIAgent",
     "SystemUIServer",
@@ -54,6 +55,7 @@ static const char *process_name_blacklist[] =
     "skhd",
     "yabai",
     "sketchybar",
+    "borders",
     "Übersicht",
     "Slack Helper (Plugin)",
     "Google Chrome Helper (Plugin)",
