@@ -128,7 +128,9 @@ void window_serialize(FILE *rsp, struct window *window)
     int space = space_manager_mission_control_index(sid);
     int display = display_arrangement(space_display_id(sid));
     int level = window_level(window->id);
+    int sub_level = window_sub_level(window->id);
     const char *layer = window_layer(level);
+    const char *sub_layer = window_layer(sub_level);
     bool is_minimized = window_is_minimized(window);
     bool visible = !is_minimized && !window->application->is_hidden && (window_check_flag(window, WINDOW_STICKY) || space_is_visible(sid));
     float opacity = window_opacity(window);
@@ -156,10 +158,13 @@ void window_serialize(FILE *rsp, struct window *window)
             "\t\"frame\":{\n\t\t\"x\":%.4f,\n\t\t\"y\":%.4f,\n\t\t\"w\":%.4f,\n\t\t\"h\":%.4f\n\t},\n"
             "\t\"role\":\"%s\",\n"
             "\t\"subrole\":\"%s\",\n"
+            "\t\"root-window\":%s,\n"
             "\t\"display\":%d,\n"
             "\t\"space\":%d,\n"
             "\t\"level\":%d,\n"
+            "\t\"sub-level\":%d,\n"
             "\t\"layer\":\"%s\",\n"
+            "\t\"sub-layer\":\"%s\",\n"
             "\t\"opacity\":%.4f,\n"
             "\t\"split-type\":\"%s\",\n"
             "\t\"split-child\":\"%s\",\n"
@@ -185,10 +190,13 @@ void window_serialize(FILE *rsp, struct window *window)
             window->frame.origin.x, window->frame.origin.y, window->frame.size.width, window->frame.size.height,
             role,
             subrole,
+            json_bool(window->is_root),
             display,
             space,
             level,
+            sub_level,
             layer,
+            sub_layer,
             opacity,
             split,
             child,
@@ -285,8 +293,8 @@ bool window_can_minimize(struct window *window)
 
 bool window_is_undersized(struct window *window)
 {
-    if (window->frame.size.width  < 200.0f) return true;
-    if (window->frame.size.height < 200.0f) return true;
+    if (window->frame.size.width  <= 500.0f) return true;
+    if (window->frame.size.height <= 500.0f) return true;
     return false;
 }
 
@@ -367,6 +375,39 @@ int window_level(uint32_t wid)
     return level;
 }
 
+static int SLSGetWindowSubLevel__Internal(int cid, uint32_t wid)
+{
+    #pragma pack(push,4)
+    struct {
+        mach_msg_header_t header;
+        NDR_record_t NDR_record;
+        uint32_t window_id;
+        int32_t sub_level;
+        int32_t padding1;
+        int32_t padding2;
+    } msg = {0};
+    #pragma pack(pop)
+
+    msg.NDR_record = NDR_record;
+    msg.window_id = wid;
+    msg.header.msgh_bits = 0x1513;
+    msg.header.msgh_remote_port = CGSGetConnectionPortById(cid);
+    msg.header.msgh_local_port = mig_get_special_reply_port();
+    msg.header.msgh_id = 0x73C3;
+    mach_msg(&msg.header, MACH_SEND_MSG|MACH_RCV_MSG, 0x24, 0x30, msg.header.msgh_local_port, 0, 0);
+
+    return msg.sub_level;
+}
+
+int window_sub_level(uint32_t wid)
+{
+    if (CGSGetConnectionPortById) {
+        return SLSGetWindowSubLevel__Internal(g_connection, wid);
+    } else {
+        return SLSGetWindowSubLevel(g_connection, wid);
+    }
+}
+
 uint64_t window_tags(struct window *window)
 {
     uint64_t tags = 0;
@@ -424,13 +465,38 @@ char *window_subrole_ts(struct window *window)
     return result;
 }
 
-bool window_level_is_standard(struct window *window)
+static bool window_is_root(struct window *window)
 {
-    int level = window_level(window->id);
-    if (level == g_layer_below_window_level)  return true;
-    if (level == g_layer_normal_window_level) return true;
-    if (level == g_layer_above_window_level)  return true;
-    return false;
+    bool result = false;
+    CFTypeRef value = NULL;
+
+    if (AXUIElementCopyAttributeValue(window->ref, kAXParentAttribute, &value) == kAXErrorSuccess) {
+        result = !(value && !CFEqual(value, window->application->ref));
+    }
+
+    if (value) CFRelease(value);
+    return result;
+}
+
+bool window_is_real(struct window *window)
+{
+    bool win = false;
+    CFStringRef role  = NULL;
+    CFStringRef srole = NULL;
+
+    if (!(role  = window_role(window)))    goto out;
+    if (!(srole = window_subrole(window))) goto role;
+
+    win = CFEqual(role, kAXWindowRole) &&
+          (CFEqual(srole, kAXStandardWindowSubrole) ||
+           CFEqual(srole, kAXFloatingWindowSubrole) ||
+           CFEqual(srole, kAXDialogSubrole));
+
+    CFRelease(srole);
+role:
+    CFRelease(role);
+out:
+    return win;
 }
 
 bool window_is_standard(struct window *window)
@@ -452,66 +518,10 @@ out:
     return standard_win;
 }
 
-bool window_is_really_a_window(struct window *window)
+bool window_level_is_standard(struct window *window)
 {
-    bool win = false;
-    CFStringRef role  = NULL;
-    CFStringRef srole = NULL;
-
-    if (!(role  = window_role(window)))    goto out;
-    if (!(srole = window_subrole(window))) goto role;
-
-    win = CFEqual(role, kAXWindowRole) &&
-          (CFEqual(srole, kAXStandardWindowSubrole) ||
-           CFEqual(srole, kAXFloatingWindowSubrole) ||
-           CFEqual(srole, kAXDialogSubrole));
-
-    CFRelease(srole);
-role:
-    CFRelease(role);
-out:
-    return win;
-}
-
-bool window_is_dialog(struct window *window)
-{
-    bool standard_win = false;
-    CFStringRef role  = NULL;
-    CFStringRef srole = NULL;
-
-    if (!(role  = window_role(window)))    goto out;
-    if (!(srole = window_subrole(window))) goto role;
-
-    standard_win = CFEqual(role, kAXWindowRole) &&
-                   CFEqual(srole, kAXDialogSubrole);
-
-    CFRelease(srole);
-role:
-    CFRelease(role);
-out:
-    return standard_win;
-}
-
-bool window_is_group(struct window *window)
-{
-    CFStringRef role = window_role(window);
-    if (!role) return false;
-
-    bool result = CFEqual(role, kAXGroupRole);
-    CFRelease(role);
-
-    return result;
-}
-
-bool window_is_popover(struct window *window)
-{
-    CFStringRef role = window_role(window);
-    if (!role) return false;
-
-    bool result = CFEqual(role, kAXPopoverRole);
-    CFRelease(role);
-
-    return result;
+    int level = window_level(window->id);
+    return level == g_layer_normal_window_level;
 }
 
 bool window_is_unknown(struct window *window)
@@ -535,6 +545,7 @@ struct window *window_create(struct application *application, AXUIElementRef win
     window->id = window_id;
     window->id_ptr = &window->id;
     window->frame = window_ax_frame(window);
+    window->is_root = window_is_root(window);
     window_set_flag(window, WINDOW_SHADOW);
 
     if (window_is_minimized(window)) {
@@ -559,3 +570,10 @@ void window_destroy(struct window *window)
     CFRelease(window->ref);
     free(window);
 }
+
+void window_dump_cg_window_levels(void)
+{
+    for (int i = 0; i < CG_WINDOW_LEVEL_KEY_COUNT; ++i) {
+        printf("CGWindowLevelForKey(%s) -> %d\n", cg_window_level_key_str[i], CGWindowLevelForKey(i));
+    }
+}
