@@ -1,6 +1,7 @@
 extern struct event_loop g_event_loop;
 extern struct process_manager g_process_manager;
 extern struct mouse_state g_mouse_state;
+extern double g_cv_host_clock_frequency;
 
 static TABLE_HASH_FUNC(hash_wm)
 {
@@ -12,6 +13,12 @@ static TABLE_COMPARE_FUNC(compare_wm)
     return *(uint32_t *) key_a == *(uint32_t *) key_b;
 }
 
+bool window_manager_is_window_eligible(struct window *window)
+{
+    bool result = window->is_root && (window_is_real(window) || window_rule_check_flag(window, WINDOW_RULE_MANAGED));
+    return result;
+}
+
 void window_manager_query_window_rules(FILE *rsp)
 {
     fprintf(rsp, "[");
@@ -115,11 +122,14 @@ void window_manager_apply_manage_rule_to_window(struct space_manager *sm, struct
     if (regex_match(rule->subrole_regex_valid, &rule->subrole_regex, window_subrole) == regex_match_subrole) return;
 
     if (rule->manage == RULE_PROP_ON) {
+        if (!rule->role_regex_valid    && !string_equals(window_role   , "AXWindow"))         return;
+        if (!rule->subrole_regex_valid && !string_equals(window_subrole, "AXStandardWindow")) return;
+
         window_rule_set_flag(window, WINDOW_RULE_MANAGED);
-        window_manager_make_window_floating(sm, wm, window, false);
+        window_manager_make_window_floating(sm, wm, window, false, true);
     } else if (rule->manage == RULE_PROP_OFF) {
         window_rule_clear_flag(window, WINDOW_RULE_MANAGED);
-        window_manager_make_window_floating(sm, wm, window, true);
+        window_manager_make_window_floating(sm, wm, window, true, true);
     }
 }
 
@@ -220,7 +230,9 @@ void window_manager_set_window_opacity_enabled(struct window_manager *wm, bool e
         while (bucket) {
             if (bucket->value) {
                 struct window *window = bucket->value;
-                window_manager_set_opacity(wm, window, enabled ? window->opacity : 1.0f);
+                if (window_manager_is_window_eligible(window)) {
+                    window_manager_set_opacity(wm, window, enabled ? window->opacity : 1.0f);
+                }
             }
 
             bucket = bucket->next;
@@ -260,16 +272,13 @@ void window_manager_center_mouse(struct window_manager *wm, struct window *windo
 
 bool window_manager_should_manage_window(struct window *window)
 {
+    if (!window->is_root)                           return false;
     if (window_check_flag(window, WINDOW_FLOAT))    return false;
     if (window_is_sticky(window))                   return false;
     if (window_check_flag(window, WINDOW_MINIMIZE)) return false;
     if (window->application->is_hidden)             return false;
 
-    if (window_rule_check_flag(window, WINDOW_RULE_MANAGED)) return true;
-
-    return ((window_level_is_standard(window)) &&
-            (window_is_standard(window)) &&
-            (window_can_move(window)));
+    return (window_is_standard(window) && window_level_is_standard(window) && window_can_move(window)) || window_rule_check_flag(window, WINDOW_RULE_MANAGED);
 }
 
 struct view *window_manager_find_managed_window(struct window_manager *wm, struct window *window)
@@ -426,8 +435,8 @@ static void window_manager_create_window_proxy(int animation_connection, struct
     SLSSetWindowOpacity(animation_connection, proxy->id, 0);
     SLSSetWindowResolution(animation_connection, proxy->id, 2.0f);
     SLSSetWindowAlpha(animation_connection, proxy->id, 1.0f);
-    SLSSetWindowLevel(animation_connection, proxy->id, g_layer_normal_window_level);
-    SLSSetWindowSubLevel(animation_connection, proxy->id, proxy->level);
+    SLSSetWindowLevel(animation_connection, proxy->id, proxy->level);
+    SLSSetWindowSubLevel(animation_connection, proxy->id, proxy->sub_level);
     proxy->context = SLWindowContextCreate(animation_connection, proxy->id, 0);
 
     CGRect frame = { {0, 0}, proxy->frame.size };
@@ -437,6 +446,15 @@ static void window_manager_create_window_proxy(int animation_connection, struct
     CFRelease(frame_region);
 }
 
+static inline void window_manager_set_window_proxy_connection_property(int animation_connection, uint32_t wid, uint32_t proxy_wid)
+{
+    CFStringRef key_ref = CFSTRINGNUM32(proxy_wid);
+    CFTypeRef value_ref = CFNUM32(wid);
+    SLSSetConnectionProperty(animation_connection, animation_connection, key_ref, value_ref);
+    CFRelease(value_ref);
+    CFRelease(key_ref);
+}
+
 static void window_manager_destroy_window_proxy(int animation_connection, struct window_proxy *proxy)
 {
     if (proxy->image) {
@@ -455,43 +473,86 @@ static void window_manager_destroy_window_proxy(int animation_connection, struct
     }
 }
 
-void *window_manager_animate_window_list_thread_proc(void *data)
+static void *window_manager_set_window_frame_thread_proc(void *data)
+{
+    struct window_animation *animation = data;
+    window_manager_set_window_frame(animation->window, animation->x, animation->y, animation->w, animation->h);
+    return NULL;
+}
+
+static void *window_manager_build_window_proxy_thread_proc(void *data)
+{
+    struct window_animation *animation = data;
+
+    animation->proxy.level = window_level(animation->wid);
+    animation->proxy.sub_level = window_sub_level(animation->wid);
+    SLSGetWindowBounds(animation->cid, animation->wid, &animation->proxy.frame);
+    animation->proxy.tx = animation->proxy.frame.origin.x;
+    animation->proxy.ty = animation->proxy.frame.origin.y;
+    animation->proxy.tw = animation->proxy.frame.size.width;
+    animation->proxy.th = animation->proxy.frame.size.height;
+    animation->proxy.image = SLSHWCaptureWindowList(animation->cid, &animation->wid, 1, (1 << 11) | (1 << 8));
+    window_manager_create_window_proxy(animation->cid, &animation->proxy);
+    window_manager_set_window_proxy_connection_property(animation->cid, animation->wid, animation->proxy.id);
+    scripting_addition_swap_window_proxy_in(animation->wid, animation->proxy.id);
+    window_manager_set_window_frame(animation->window, animation->x, animation->y, animation->w, animation->h);
+
+    return NULL;
+}
+
+static CVReturn window_manager_animate_window_list_thread_proc(CVDisplayLinkRef link, const CVTimeStamp *now, const CVTimeStamp *output_time, CVOptionFlags flags, CVOptionFlags *flags_out, void *data)
 {
     struct window_animation_context *context = data;
-    int animation_count = buf_len(context->animation_list);
+    int animation_count = context->animation_count;
 
-    ANIMATE(context->animation_connection, context->animation_frame_rate, context->animation_duration, ease_out_cubic, {
-        for (int i = 0; i < animation_count; ++i) {
-            if (context->animation_list[i].skip) continue;
+    uint64_t current_clock = output_time->hostTime;
+    if (!context->animation_clock) context->animation_clock = now->hostTime;
 
-            context->animation_list[i].proxy.tx = lerp(context->animation_list[i].proxy.frame.origin.x,    mt, context->animation_list[i].x);
-            context->animation_list[i].proxy.ty = lerp(context->animation_list[i].proxy.frame.origin.y,    mt, context->animation_list[i].y);
-            context->animation_list[i].proxy.tw = lerp(context->animation_list[i].proxy.frame.size.width,  mt, context->animation_list[i].w);
-            context->animation_list[i].proxy.th = lerp(context->animation_list[i].proxy.frame.size.height, mt, context->animation_list[i].h);
+    double t = (double)(current_clock - context->animation_clock) / (double)(context->animation_duration * g_cv_host_clock_frequency);
+    if (t <= 0.0) t = 0.0f;
+    if (t >= 1.0) t = 1.0f;
 
-            CGAffineTransform transform = CGAffineTransformMakeTranslation(-context->animation_list[i].proxy.tx, -context->animation_list[i].proxy.ty);
-            CGAffineTransform scale = CGAffineTransformMakeScale(context->animation_list[i].proxy.frame.size.width / context->animation_list[i].proxy.tw, context->animation_list[i].proxy.frame.size.height / context->animation_list[i].proxy.th);
-            SLSTransactionSetWindowTransform(transaction, context->animation_list[i].proxy.id, 0, 0, CGAffineTransformConcat(transform, scale));
-        }
-    });
+    float mt = ease_out_cubic(t);
+    CFTypeRef transaction = SLSTransactionCreate(context->animation_connection);
+
+    for (int i = 0; i < animation_count; ++i) {
+        if (__atomic_load_n(&context->animation_list[i].skip, __ATOMIC_RELAXED)) continue;
+
+        context->animation_list[i].proxy.tx = lerp(context->animation_list[i].proxy.frame.origin.x,    mt, context->animation_list[i].x);
+        context->animation_list[i].proxy.ty = lerp(context->animation_list[i].proxy.frame.origin.y,    mt, context->animation_list[i].y);
+        context->animation_list[i].proxy.tw = lerp(context->animation_list[i].proxy.frame.size.width,  mt, context->animation_list[i].w);
+        context->animation_list[i].proxy.th = lerp(context->animation_list[i].proxy.frame.size.height, mt, context->animation_list[i].h);
+
+        CGAffineTransform transform = CGAffineTransformMakeTranslation(-context->animation_list[i].proxy.tx, -context->animation_list[i].proxy.ty);
+        CGAffineTransform scale = CGAffineTransformMakeScale(context->animation_list[i].proxy.frame.size.width / context->animation_list[i].proxy.tw, context->animation_list[i].proxy.frame.size.height / context->animation_list[i].proxy.th);
+        SLSTransactionSetWindowTransform(transaction, context->animation_list[i].proxy.id, 0, 0, CGAffineTransformConcat(transform, scale));
+    }
+
+    SLSTransactionCommit(transaction, 0);
+    CFRelease(transaction);
+    if (t != 1.0f) goto out;
 
     pthread_mutex_lock(&g_window_manager.window_animations_lock);
     SLSDisableUpdate(context->animation_connection);
     for (int i = 0; i < animation_count; ++i) {
-        if (context->animation_list[i].skip) continue;
+        if (__atomic_load_n(&context->animation_list[i].skip, __ATOMIC_RELAXED)) continue;
 
-        table_remove(&g_window_manager.window_animations_table, &context->animation_list[i].wid);
         scripting_addition_swap_window_proxy_out(context->animation_list[i].wid, context->animation_list[i].proxy.id);
+        table_remove(&g_window_manager.window_animations_table, &context->animation_list[i].wid);
         window_manager_destroy_window_proxy(context->animation_connection, &context->animation_list[i].proxy);
     }
     SLSReenableUpdate(context->animation_connection);
     pthread_mutex_unlock(&g_window_manager.window_animations_lock);
 
     SLSReleaseConnection(context->animation_connection);
-    buf_free(context->animation_list);
-
+    free(context->animation_list);
     free(context);
-    return NULL;
+
+    CVDisplayLinkStop(link);
+    CVDisplayLinkRelease(link);
+
+out:
+    return kCVReturnSuccess;
 }
 
 void window_manager_animate_window_list_async(struct window_capture *window_list, int window_count)
@@ -499,41 +560,59 @@ void window_manager_animate_window_list_async(struct window_capture *window_list
     struct window_animation_context *context = malloc(sizeof(struct window_animation_context));
 
     SLSNewConnection(0, &context->animation_connection);
+    context->animation_count    = window_count;
+    context->animation_list     = malloc(window_count * sizeof(struct window_animation));
     context->animation_duration = g_window_manager.window_animation_duration;
-    context->animation_frame_rate = g_window_manager.window_animation_frame_rate;
-    context->animation_list = NULL;
+    context->animation_clock    = 0;
 
     for (int i = 0; i < window_count; ++i) {
         struct window_capture *capture = &window_list[i];
 
-        buf_push(context->animation_list, ((struct window_animation) {
-            .wid   = capture->window->id,
-            .x     = capture->x,
-            .y     = capture->y,
-            .w     = capture->w,
-            .h     = capture->h,
-            .proxy = {0},
-            .skip  = false
-        }));
+        context->animation_list[i] = (struct window_animation) {
+            .window = capture->window,
+            .wid    = capture->window->id,
+            .x      = capture->x,
+            .y      = capture->y,
+            .w      = capture->w,
+            .h      = capture->h,
+            .cid    = context->animation_connection,
+            .proxy  = {0},
+            .skip   = false
+        };
     }
 
+    int thread_count = 0;
+    pthread_t *threads = ts_alloc_list(pthread_t, window_count);
+
     pthread_mutex_lock(&g_window_manager.window_animations_lock);
     SLSDisableUpdate(context->animation_connection);
-    for (int i = 0; i < buf_len(context->animation_list); ++i) {
+    for (int i = 0; i < window_count; ++i) {
         struct window_animation *existing_animation = table_find(&g_window_manager.window_animations_table, &context->animation_list[i].wid);
         if (existing_animation) {
-            table_remove(&g_window_manager.window_animations_table, &context->animation_list[i].wid);
+            __atomic_store_n(&existing_animation->skip, true, __ATOMIC_RELEASE);
+
+            pthread_t thread;
+            if (pthread_create(&thread, NULL, &window_manager_set_window_frame_thread_proc, &context->animation_list[i]) == 0) {
+                pthread_detach(thread);
+            } else {
+                window_manager_set_window_frame_thread_proc(&context->animation_list[i]);
+            }
 
-            existing_animation->skip = true;
             context->animation_list[i].proxy.frame.origin.x    = (int)(existing_animation->proxy.tx);
             context->animation_list[i].proxy.frame.origin.y    = (int)(existing_animation->proxy.ty);
             context->animation_list[i].proxy.frame.size.width  = (int)(existing_animation->proxy.tw);
             context->animation_list[i].proxy.frame.size.height = (int)(existing_animation->proxy.th);
+            context->animation_list[i].proxy.tx                = (int)(existing_animation->proxy.tx);
+            context->animation_list[i].proxy.ty                = (int)(existing_animation->proxy.ty);
+            context->animation_list[i].proxy.tw                = (int)(existing_animation->proxy.tw);
+            context->animation_list[i].proxy.th                = (int)(existing_animation->proxy.th);
             context->animation_list[i].proxy.level             = existing_animation->proxy.level;
+            context->animation_list[i].proxy.sub_level         = existing_animation->proxy.sub_level;
             context->animation_list[i].proxy.image             = CFRetain(existing_animation->proxy.image);
             __asm__ __volatile__ ("" ::: "memory");
 
             window_manager_create_window_proxy(context->animation_connection, &context->animation_list[i].proxy);
+            window_manager_set_window_proxy_connection_property(context->animation_connection, context->animation_list[i].wid, context->animation_list[i].proxy.id);
 
             CFTypeRef transaction1 = SLSTransactionCreate(context->animation_connection);
             CFTypeRef transaction2 = SLSTransactionCreate(context->animation_connection);
@@ -542,33 +621,37 @@ void window_manager_animate_window_list_async(struct window_capture *window_list
             SLSTransactionSetWindowSystemAlpha(transaction2, existing_animation->proxy.id, 0);
 
             SLSTransactionCommit(transaction1, 1);
-            usleep(10000);
+            usleep(15000);
             SLSTransactionCommit(transaction2, 1);
 
             CFRelease(transaction1);
             CFRelease(transaction2);
 
-            window_manager_destroy_window_proxy(context->animation_connection, &existing_animation->proxy);
+            table_remove(&g_window_manager.window_animations_table, &context->animation_list[i].wid);
+            window_manager_destroy_window_proxy(existing_animation->cid, &existing_animation->proxy);
         } else {
-            context->animation_list[i].proxy.level = window_level(context->animation_list[i].wid);
-            SLSGetWindowBounds(context->animation_connection, context->animation_list[i].wid, &context->animation_list[i].proxy.frame);
-            context->animation_list[i].proxy.image = SLSHWCaptureWindowList(context->animation_connection, &context->animation_list[i].wid, 1, (1 << 11) | (1 << 8));
-            window_manager_create_window_proxy(context->animation_connection, &context->animation_list[i].proxy);
-            scripting_addition_swap_window_proxy_in(context->animation_list[i].wid, context->animation_list[i].proxy.id);
+            pthread_t thread;
+            if (pthread_create(&thread, NULL, &window_manager_build_window_proxy_thread_proc, &context->animation_list[i]) == 0) {
+                threads[thread_count++] = thread;
+            } else {
+                window_manager_build_window_proxy_thread_proc(&context->animation_list[i]);
+            }
         }
 
         table_add(&g_window_manager.window_animations_table, &context->animation_list[i].wid, &context->animation_list[i]);
     }
+
+    for (int i = 0; i < thread_count; ++i) {
+        pthread_join(threads[i], NULL);
+    }
+
     SLSReenableUpdate(context->animation_connection);
     pthread_mutex_unlock(&g_window_manager.window_animations_lock);
 
-    for (int i = 0; i < window_count; ++i) {
-        window_manager_set_window_frame(window_list[i].window, window_list[i].x, window_list[i].y, window_list[i].w, window_list[i].h);
-    }
-
-    pthread_t thread;
-    pthread_create(&thread, NULL, &window_manager_animate_window_list_thread_proc, context);
-    pthread_detach(thread);
+    CVDisplayLinkRef link;
+    CVDisplayLinkCreateWithActiveCGDisplays(&link);
+    CVDisplayLinkSetOutputCallback(link, window_manager_animate_window_list_thread_proc, context);
+    CVDisplayLinkStart(link);
 }
 
 void window_manager_animate_window_list(struct window_capture *window_list, int window_count)
@@ -622,7 +705,9 @@ void window_manager_set_purify_mode(struct window_manager *wm, enum purify_mode
         while (bucket) {
             if (bucket->value) {
                 struct window *window = bucket->value;
-                window_manager_purify_window(wm, window);
+                if (window_manager_is_window_eligible(window)) {
+                    window_manager_purify_window(wm, window);
+                }
             }
 
             bucket = bucket->next;
@@ -645,14 +730,9 @@ bool window_manager_set_opacity(struct window_manager *wm, struct window *window
 
 void window_manager_set_window_opacity(struct window_manager *wm, struct window *window, float opacity)
 {
-    if (!wm->enable_window_opacity) return;
-    if (window->opacity != 0.0f)    return;
-
-    if ((!window_rule_check_flag(window, WINDOW_RULE_MANAGED)) &&
-        (!window_is_standard(window)) &&
-        (!window_is_dialog(window))) {
-        return;
-    }
+    if (!wm->enable_window_opacity)                 return;
+    if (!window_manager_is_window_eligible(window)) return;
+    if (window->opacity != 0.0f)                    return;
 
     window_manager_set_opacity(wm, window, opacity);
 }
@@ -673,7 +753,9 @@ void window_manager_set_normal_window_opacity(struct window_manager *wm, float o
             if (bucket->value) {
                 struct window *window = bucket->value;
                 if (window->id == wm->focused_window_id) goto next;
-                window_manager_set_window_opacity(wm, window, wm->normal_window_opacity);
+                if (window_manager_is_window_eligible(window)) {
+                    window_manager_set_window_opacity(wm, window, wm->normal_window_opacity);
+                }
             }
 
 next:
@@ -795,6 +877,17 @@ struct window *window_manager_find_window_on_space_by_rank_filtering_window(stru
     return result;
 }
 
+static inline bool window_manager_window_connection_is_jankyborders(int window_cid)
+{
+    static char process_name[PROC_PIDPATHINFO_MAXSIZE];
+
+    pid_t window_pid = 0;
+    SLSConnectionGetPID(window_cid, &window_pid);
+    proc_name(window_pid, process_name, sizeof(process_name));
+
+    return strcmp(process_name, "borders") == 0;
+}
+
 struct window *window_manager_find_window_at_point_filtering_window(struct window_manager *wm, CGPoint point, uint32_t filter_wid)
 {
     CGPoint window_point;
@@ -802,7 +895,10 @@ struct window *window_manager_find_window_at_point_filtering_window(struct windo
     int window_cid;
 
     SLSFindWindowByGeometry(g_connection, filter_wid, -1, 0, &point, &window_point, &window_id, &window_cid);
-    if (g_connection == window_cid) SLSFindWindowByGeometry(g_connection, window_id, -1, 0, &point, &window_point, &window_id, &window_cid);
+
+    if (window_manager_window_connection_is_jankyborders(window_cid)) {
+        SLSFindWindowByGeometry(g_connection, window_id, -1, 0, &point, &window_point, &window_id, &window_cid);
+    }
 
     return window_manager_find_window(wm, window_id);
 }
@@ -814,7 +910,10 @@ struct window *window_manager_find_window_at_point(struct window_manager *wm, CG
     int window_cid;
 
     SLSFindWindowByGeometry(g_connection, 0, 1, 0, &point, &window_point, &window_id, &window_cid);
-    if (g_connection == window_cid) SLSFindWindowByGeometry(g_connection, window_id, -1, 0, &point, &window_point, &window_id, &window_cid);
+
+    if (window_manager_window_connection_is_jankyborders(window_cid)) {
+        SLSFindWindowByGeometry(g_connection, window_id, -1, 0, &point, &window_point, &window_id, &window_cid);
+    }
 
     return window_manager_find_window(wm, window_id);
 }
@@ -1277,7 +1376,7 @@ struct window *window_manager_create_and_add_window(struct space_manager *sm, st
     char *window_title = window_title_ts(window);
     char *window_role = window_role_ts(window);
     char *window_subrole = window_subrole_ts(window);
-    debug("%s:%d %s - %s (%s:%s)\n", __FUNCTION__, window->id, window->application->name, window_title, window_role, window_subrole);
+    debug("%s:%d %s - %s (%s:%s:%d)\n", __FUNCTION__, window->id, window->application->name, window_title, window_role, window_subrole, window->is_root);
 
     if (window_is_unknown(window)) {
         debug("%s: ignoring AXUnknown window %s %d\n", __FUNCTION__, window->application->name, window->id);
@@ -1286,48 +1385,9 @@ struct window *window_manager_create_and_add_window(struct space_manager *sm, st
         return NULL;
     }
 
-    if (window_is_popover(window)) {
-        debug("%s: ignoring AXPopover window %s %d\n", __FUNCTION__, window->application->name, window->id);
-        window_manager_remove_lost_focused_event(wm, window->id);
-        window_destroy(window);
-        return NULL;
-    }
-
-    if (window_is_group(window)) {
-        debug("%s: ignoring AXGroup window %s %d\n", __FUNCTION__, window->application->name, window->id);
-        window_manager_remove_lost_focused_event(wm, window->id);
-        window_destroy(window);
-        return NULL;
-    }
-
     //
-    // NOTE(koekeishiya): A lot of windows misreport their accessibility role, so we allow the user
-    // to specify rules to make sure that we do in fact manage these windows properly.
+    // NOTE(koekeishiya): Attempt to track **all** windows.
     //
-    // This part of the rule must be applied at this stage (prior to other rule properties), and if
-    // no such rule matches this window, it will be ignored if it does not have a role of kAXWindowRole.
-    //
-
-    window_manager_apply_manage_rules_to_window(sm, wm, window, window_title, window_role, window_subrole);
-
-    if (!window_is_really_a_window(window) && !window_rule_check_flag(window, WINDOW_RULE_MANAGED)) {
-        debug("%s ignoring incorrectly marked window %s %d\n", __FUNCTION__, window->application->name, window->id);
-
-        //
-        // NOTE(koekeishiya): Print window information when debug_output is enabled.
-        // Useful for identifying and creating rules if this window should in fact be managed.
-        //
-
-        if (g_verbose) {
-            fprintf(stdout, "window info: \n");
-            window_serialize(stdout, window);
-            fprintf(stdout, "\n");
-        }
-
-        window_manager_remove_lost_focused_event(wm, window->id);
-        window_destroy(window);
-        return NULL;
-    }
 
     if (!window_observe(window)) {
         debug("%s: could not observe %s %d\n", __FUNCTION__, window->application->name, window->id);
@@ -1342,26 +1402,78 @@ struct window *window_manager_create_and_add_window(struct space_manager *sm, st
         window_manager_remove_lost_focused_event(wm, window->id);
     }
 
-    window_manager_apply_rules_to_window(sm, wm, window, window_title, window_role, window_subrole);
     window_manager_add_window(wm, window);
-    window_manager_purify_window(wm, window);
-    window_manager_set_window_opacity(wm, window, wm->normal_window_opacity);
 
-    if ((!application->is_hidden) &&
-        (!window_check_flag(window, WINDOW_MINIMIZE)) &&
-        (!window_check_flag(window, WINDOW_FULLSCREEN)) &&
-        (!window_rule_check_flag(window, WINDOW_RULE_MANAGED))) {
-        if (window_rule_check_flag(window, WINDOW_RULE_FULLSCREEN)) {
-            window_rule_clear_flag(window, WINDOW_RULE_FULLSCREEN);
-        } else if ((!window_level_is_standard(window)) ||
-                   (!window_is_standard(window)) ||
-                   (!window_can_move(window)) ||
-                   (window_is_sticky(window)) ||
-                   (!window_can_resize(window) && window_is_undersized(window))) {
+    //
+    // NOTE(koekeishiya): However, only **root windows** are eligible for management.
+    //
+
+    if (window->is_root) {
+
+        //
+        // NOTE(koekeishiya): A lot of windows misreport their accessibility role, so we allow the user
+        // to specify rules to make sure that we do in fact manage these windows properly.
+        //
+        // This part of the rule must be applied at this stage (prior to other rule properties), and if
+        // no such rule matches this window, it will be ignored if it does not have a role of kAXWindowRole.
+        //
+
+        window_manager_apply_manage_rules_to_window(sm, wm, window, window_title, window_role, window_subrole);
+
+        if (window_manager_is_window_eligible(window)) {
+            window->is_eligible = true;
+            window_manager_apply_rules_to_window(sm, wm, window, window_title, window_role, window_subrole);
+            window_manager_purify_window(wm, window);
+            window_manager_set_window_opacity(wm, window, wm->normal_window_opacity);
+
+            if (application->is_hidden)                              goto out;
+            if (window_check_flag(window, WINDOW_MINIMIZE))          goto out;
+            if (window_check_flag(window, WINDOW_FULLSCREEN))        goto out;
+            if (window_rule_check_flag(window, WINDOW_RULE_MANAGED)) goto out;
+
+            if (window_rule_check_flag(window, WINDOW_RULE_FULLSCREEN)) {
+                window_rule_clear_flag(window, WINDOW_RULE_FULLSCREEN);
+                goto out;
+            }
+
+            if (window_is_sticky(window) ||
+                !window_can_move(window) ||
+                !window_is_standard(window) ||
+                !window_level_is_standard(window) ||
+                (!window_can_resize(window) && window_is_undersized(window))) {
+                window_set_flag(window, WINDOW_FLOAT);
+            }
+        } else {
+            debug("%s ignoring incorrectly marked window %s %d\n", __FUNCTION__, window->application->name, window->id);
             window_set_flag(window, WINDOW_FLOAT);
+
+            //
+            // NOTE(koekeishiya): Print window information when debug_output is enabled.
+            // Useful for identifying and creating rules if this window should in fact be managed.
+            //
+
+            if (g_verbose) {
+                fprintf(stdout, "window info: \n");
+                window_serialize(stdout, window);
+                fprintf(stdout, "\n");
+            }
+        }
+    } else {
+        debug("%s ignoring child window %s %d\n", __FUNCTION__, window->application->name, window->id);
+        window_set_flag(window, WINDOW_FLOAT);
+
+        //
+        // NOTE(koekeishiya): Print window information when debug_output is enabled.
+        //
+
+        if (g_verbose) {
+            fprintf(stdout, "window info: \n");
+            window_serialize(stdout, window);
+            fprintf(stdout, "\n");
         }
     }
 
+out:
     return window;
 }
 
@@ -1421,7 +1533,6 @@ void window_manager_add_existing_application_windows(struct space_manager *sm, s
     uint32_t *global_window_list = window_manager_existing_application_window_list(application, &global_window_count);
     if (!global_window_list) return;
 
-
     CFArrayRef window_list_ref = application_window_list(application);
     int window_count = window_list_ref ? CFArrayGetCount(window_list_ref) : 0;
 
@@ -1539,6 +1650,7 @@ enum window_op_error window_manager_stack_window(struct space_manager *sm, struc
         window_manager_remove_managed_window(wm, b->id);
         window_manager_purify_window(wm, b);
     } else if (window_check_flag(b, WINDOW_FLOAT)) {
+        if (!window_manager_is_window_eligible(b)) return WINDOW_OP_ERROR_INVALID_SRC_NODE;
         window_clear_flag(b, WINDOW_FLOAT);
         if (window_check_flag(b, WINDOW_STICKY)) window_manager_make_window_sticky(sm, wm, b, false);
     }
@@ -1872,8 +1984,18 @@ enum window_op_error window_manager_apply_grid(struct space_manager *sm, struct
     return WINDOW_OP_ERROR_SUCCESS;
 }
 
-void window_manager_make_window_floating(struct space_manager *sm, struct window_manager *wm, struct window *window, bool should_float)
+void window_manager_make_window_floating(struct space_manager *sm, struct window_manager *wm, struct window *window, bool should_float, bool force)
 {
+    if (!window_manager_is_window_eligible(window)) return;
+
+    if (!force) {
+        if (!window_is_standard(window) || !window_level_is_standard(window) || !window_can_move(window)) {
+            if (!window_rule_check_flag(window, WINDOW_RULE_MANAGED)) {
+                return;
+            }
+        }
+    }
+
     if (should_float) {
         struct view *view = window_manager_find_managed_window(wm, window);
         if (view) {
@@ -1896,6 +2018,8 @@ void window_manager_make_window_floating(struct space_manager *sm, struct window
 
 void window_manager_make_window_sticky(struct space_manager *sm, struct window_manager *wm, struct window *window, bool should_sticky)
 {
+    if (!window_manager_is_window_eligible(window)) return;
+
     if (should_sticky) {
         if (scripting_addition_set_sticky(window->id, true)) {
             struct view *view = window_manager_find_managed_window(wm, window);
@@ -2227,7 +2351,6 @@ void window_manager_init(struct window_manager *wm)
     wm->normal_window_opacity = 1.0f;
     wm->window_opacity_duration = 0.0f;
     wm->window_animation_duration = 0.0f;
-    wm->window_animation_frame_rate = 120;
     wm->insert_feedback_windows = NULL;
     wm->insert_feedback_color = rgba_color_from_hex(0xffd75f5f);
 
