@@ -89,7 +89,6 @@ struct window_manager
     float normal_window_opacity;
     float window_opacity_duration;
     float window_animation_duration;
-    int window_animation_frame_rate;
     uint32_t *insert_feedback_windows;
     struct rgba_color insert_feedback_color;
 };
@@ -103,6 +102,7 @@ void window_manager_apply_rule_to_window(struct space_manager *sm, struct window
 void window_manager_apply_manage_rules_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, char *window_title, char *window_role, char *window_subrole);
 void window_manager_apply_rules_to_window(struct space_manager *sm, struct window_manager *wm, struct window *window, char *window_title, char *window_role, char *window_subrole);
 void window_manager_center_mouse(struct window_manager *wm, struct window *window);
+bool window_manager_is_window_eligible(struct window *window);
 bool window_manager_should_manage_window(struct window *window);
 void window_manager_tile_window(struct window_manager *wm, struct window *window);
 void window_manager_move_window(struct window *window, float x, float y);
@@ -178,7 +178,7 @@ struct window **window_manager_add_application_windows(struct space_manager *sm,
 void window_manager_add_existing_application_windows(struct space_manager *sm, struct window_manager *wm, struct application *application, int refresh_index);
 enum window_op_error window_manager_apply_grid(struct space_manager *sm, struct window_manager *wm, struct window *window, unsigned r, unsigned c, unsigned x, unsigned y, unsigned w, unsigned h);
 void window_manager_purify_window(struct window_manager *wm, struct window *window);
-void window_manager_make_window_floating(struct space_manager *sm, struct window_manager *wm, struct window *window, bool should_float);
+void window_manager_make_window_floating(struct space_manager *sm, struct window_manager *wm, struct window *window, bool should_float, bool force);
 void window_manager_make_window_sticky(struct space_manager *sm, struct window_manager *wm, struct window *window, bool should_sticky);
 void window_manager_adjust_layer(struct window *window, int layer);
 bool window_manager_set_window_layer(struct window *window, int layer);
