@@ -22,7 +22,7 @@
 
 #define MAJOR  6
 #define MINOR  0
-#define PATCH  4
+#define PATCH 10
 
 struct event_loop g_event_loop;
 void *g_workspace_context;
@@ -31,6 +31,7 @@ struct display_manager g_display_manager;
 struct space_manager g_space_manager;
 struct window_manager g_window_manager;
 struct mouse_state g_mouse_state;
+double g_cv_host_clock_frequency;
 int g_layer_normal_window_level;
 int g_layer_below_window_level;
 int g_layer_above_window_level;
@@ -177,9 +178,11 @@ static inline bool configure_settings_and_acquire_lock(void)
 
     g_pid = getpid();
     g_connection = SLSMainConnectionID();
+    g_cv_host_clock_frequency   = CVGetHostClockFrequency();
     g_layer_normal_window_level = CGWindowLevelForKey(LAYER_NORMAL);
     g_layer_below_window_level  = CGWindowLevelForKey(LAYER_BELOW);
     g_layer_above_window_level  = CGWindowLevelForKey(LAYER_ABOVE);
+    CGSGetConnectionPortById    = macho_find_symbol("/System/Library/PrivateFrameworks/SkyLight.framework/Versions/A/SkyLight", "_CGSGetConnectionPortById");
 
     NSApplicationLoad();
     signal(SIGCHLD, SIG_IGN);
@@ -192,6 +195,7 @@ static inline bool configure_settings_and_acquire_lock(void)
     hook_nsobject_autorelease();
     hook_autoreleasepool_drain();
     hook_autoreleasepool_release();
+    window_dump_cg_window_levels();
 #endif
 
     int handle = open(g_lock_file, O_CREAT | O_WRONLY, 0600);
